@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from collections import OrderedDict
 from decimal import Decimal
 from typing import Optional, Tuple
@@ -10,13 +12,17 @@ from django.http import HttpRequest
 from django.utils.text import slugify
 from django.utils.translation import gettext_lazy as _
 
-from salesman.core.models import JSONField
+from salesman.conf import app_settings
+from salesman.core.utils import get_salesman_model
 
 BASKET_ID_SESSION_KEY = 'BASKET_ID'
 
 
 class BasketManager(models.Manager):
-    def get_or_create_from_request(self, request: HttpRequest) -> Tuple['Basket', bool]:
+    def get_or_create_from_request(
+        self,
+        request: HttpRequest,
+    ) -> Tuple[BaseBasket, bool]:
         """
         Get basket from request or create a new one.
         If user is logged in session basket gets merged into a user basket.
@@ -29,7 +35,7 @@ class BasketManager(models.Manager):
         try:
             session_basket_id = request.session[BASKET_ID_SESSION_KEY]
             session_basket = self.get(id=session_basket_id, owner=None)
-        except (KeyError, Basket.DoesNotExist):
+        except (KeyError, self.model.DoesNotExist):
             session_basket = None
 
         if hasattr(request, 'user') and request.user.is_authenticated:
@@ -57,7 +63,7 @@ class BasketManager(models.Manager):
         return basket, created
 
 
-class Basket(models.Model):
+class BaseBasket(models.Model):
     owner = models.ForeignKey(
         settings.AUTH_USER_MODEL,
         on_delete=models.CASCADE,
@@ -65,16 +71,17 @@ class Basket(models.Model):
         verbose_name=_("Owner"),
     )
 
-    extra = JSONField(_("Extra"), blank=True)
+    extra = models.JSONField(_("Extra"), blank=True, default=dict)
 
     date_created = models.DateTimeField(_("Date created"), auto_now_add=True)
     date_updated = models.DateTimeField(_("Date updated"), auto_now=True)
 
     objects = BasketManager()
 
-    _cached_items = None
+    _cached_items: Optional[list[BaseBasketItem]] = None
 
     class Meta:
+        abstract = True
         verbose_name = _("Basket")
         verbose_name_plural = _("Baskets")
         ordering = ['-date_created']
@@ -99,14 +106,33 @@ class Basket(models.Model):
         from .modifiers import basket_modifiers_pool
 
         items = self.get_items()
-        self.extra_rows = OrderedDict()
-        self.subtotal = 0
+
+        # Setup basket and items.
+        for modifier in basket_modifiers_pool.get_modifiers():
+            modifier.setup_basket(self, request)
+            for item in items:
+                modifier.setup_item(item, request)
+
+        self.extra_rows: dict = OrderedDict()
+        self.subtotal = Decimal(0)
+        self.total = Decimal(0)
+
+        # Process basket items.
         for item in items:
             item.update(request)
             self.subtotal += item.total
         self.total = self.subtotal
+
+        # Finalize items and process basket.
         for modifier in basket_modifiers_pool.get_modifiers():
+            for item in items:
+                modifier.finalize_item(item, request)
             modifier.process_basket(self, request)
+
+        # Finalize basket.
+        for modifier in basket_modifiers_pool.get_modifiers():
+            modifier.finalize_basket(self, request)
+
         self._cached_items = items
 
     def add(
@@ -115,13 +141,14 @@ class Basket(models.Model):
         quantity: int = 1,
         ref: Optional[str] = None,
         extra: Optional[dict] = None,
-    ) -> 'BasketItem':
+    ) -> BaseBasketItem:
         """
         Add product to the basket.
 
         Returns:
             BasketItem: BasketItem instance
         """
+        BasketItem = get_salesman_model('BasketItem')
         if not ref:
             ref = BasketItem.get_product_ref(product)
         try:
@@ -145,13 +172,29 @@ class Basket(models.Model):
         Remove item with given ``ref`` from the basket.
 
         Args:
-            ref (str): Item ref to remove.
+            ref (str): Item ref to remove
         """
-        try:
-            self.items.get(ref=ref).delete()
+        item = self.find(ref)
+        if item:
+            item.delete()
             self._cached_items = None
-        except BasketItem.DoesNotExist:
-            pass
+
+    def find(self, ref: str) -> Optional[BaseBasketItem]:
+        """
+        Find item with given ``ref`` in the basket.
+
+        Args:
+            ref (str): Item ref
+
+        Returns:
+            Optional[BaseBasketItem]: Basket item if found.
+        """
+        if self._cached_items is not None:
+            try:
+                return [item for item in self._cached_items if item.ref == ref][0]
+            except IndexError:
+                return None
+        return self.items.filter(ref=ref).first()
 
     def clear(self) -> None:
         """
@@ -161,7 +204,7 @@ class Basket(models.Model):
         self._cached_items = None
 
     @transaction.atomic
-    def merge(self, other: 'Basket') -> None:
+    def merge(self, other: BaseBasket) -> None:
         """
         Merge other basket with this one, delete afterwards.
 
@@ -179,12 +222,12 @@ class Basket(models.Model):
         other.delete()
         self._cached_items = None
 
-    def get_items(self) -> list:
+    def get_items(self) -> list[BaseBasketItem]:
         """
         Returns items from cache or stores new ones.
         """
         if self._cached_items is None:
-            self._cached_items = list(self.items.all().prefetch_related("product"))
+            self._cached_items = list(self.items.all().prefetch_related('product'))
         return self._cached_items
 
     @property
@@ -207,9 +250,18 @@ class Basket(models.Model):
         return aggr['quantity'] or 0
 
 
-class BasketItem(models.Model):
+class Basket(BaseBasket):
+    """
+    Model that can be swapped by overriding `SALESMAN_BASKET_MODEL` setting.
+    """
+
+    class Meta(BaseBasket.Meta):
+        swappable = 'SALESMAN_BASKET_MODEL'
+
+
+class BaseBasketItem(models.Model):
     basket = models.ForeignKey(
-        Basket,
+        app_settings.SALESMAN_BASKET_MODEL,
         on_delete=models.CASCADE,
         related_name='items',
         verbose_name=_("Basket"),
@@ -224,12 +276,13 @@ class BasketItem(models.Model):
     product = GenericForeignKey('product_content_type', 'product_id')
 
     quantity = models.PositiveIntegerField(_("Quantity"), default=1)
-    extra = JSONField(_("Extra"), blank=True)
+    extra = models.JSONField(_("Extra"), blank=True, default=dict)
 
     date_created = models.DateTimeField(_("Date created"), auto_now_add=True)
     date_updated = models.DateTimeField(_("Date updated"), auto_now=True)
 
     class Meta:
+        abstract = True
         verbose_name = _("Item")
         verbose_name_plural = _("Items")
         unique_together = ('basket', 'ref')
@@ -256,10 +309,11 @@ class BasketItem(models.Model):
         """
         from .modifiers import basket_modifiers_pool
 
-        self.extra_rows = OrderedDict()
+        self.extra_rows: dict = OrderedDict()
         self.unit_price = Decimal(self.product.get_price(request))
         self.subtotal = self.unit_price * self.quantity
         self.total = self.subtotal
+
         for modifier in basket_modifiers_pool.get_modifiers():
             modifier.process_item(self, request)
 
@@ -278,14 +332,23 @@ class BasketItem(models.Model):
         return self.product.code
 
     @classmethod
-    def get_product_ref(cls, product: object) -> str:
+    def get_product_ref(cls, product: models.Model) -> str:
         """
         Returns default item ``ref`` for given product.
 
         Args:
-            product (object): Product instance
+            product (models.Model): Product instance
 
         Returns:
             str: Item ref
         """
         return slugify(f'{product._meta.label}-{product.id}')
+
+
+class BasketItem(BaseBasketItem):
+    """
+    Model that can be swapped by overriding `SALESMAN_BASKET_ITEM_MODEL` setting.
+    """
+
+    class Meta(BaseBasketItem.Meta):
+        swappable = 'SALESMAN_BASKET_ITEM_MODEL'
